/* Demonstrates how to register a C function ("builtin") in a script context */

#include <estutter/stutter.h>
#include <stdio.h>

static STT_BUILTIN(sb_hello_world)
{
	printf("Hello World!\n");
	return STT_NIL(ctx->stt);
}

int main()
{
	stt_ctx_t stt;
	stt_varctx_t *ctx;

	/* initialize the lib */
	stt_init(&stt);

	/* initialize execution context - keep it vanilla, no builtins, no globals */
	ctx = stt_varctx_create(&stt, NULL, 1024);

	/* register the new builtin */
	stt_varctx_set(ctx, "HELLO-WORLD", stt_create_builtin(&stt, sb_hello_world));

	/* eval a script that calls it */
	stt_evalstr(ctx, "(HELLO-WORLD)\n");

	/* clean up */
	stt_varctx_free(ctx);
	stt_uninit(&stt);
	return 0;
}
