static STT_BUILTIN(sb_toblob)
{
	stt_obj_t *from;

	STT_ARG_OBJ(ctx->stt, from);
	if(from->type == STT_ST_STRING) {
		int l = strlen(from->d.string.value);
		return stt_create_blob_copy(ctx->stt, (unsigned char *) from->d.string.value, l);
	} else if(from->type == STT_ST_CHAR) {
		return stt_create_blob_copy(ctx->stt, &from->d.ch.ch, 1);
	} else {
		return stt_create_error(ctx->stt, STT_SE_TYPE, "->BLOB: invalid argument (want string or char)");
	}
}

static STT_BUILTIN(sb_blob_dump)
{
	stt_obj_t *blob;
	STT_ARG_BLOB(ctx->stt, blob);
	return stt_stream_dumpbuf(ctx, blob->d.blob.data, blob->d.blob.size);
}

static STT_BUILTIN(sb_blob_read)
{
	long bytes;
	size_t readlen;
	unsigned char *buf;
	stt_obj_t *o;
	STT_L_INT(ctx->stt, bytes);

	if(bytes < 1) return stt_create_error(ctx->stt, STT_SE_PARM, "blob-read: bytes must be > 0");

	buf = (unsigned char *) stt_malloc(bytes);
	o = stt_stream_readbuf(ctx, buf, bytes, &readlen);
	if(o->type == STT_ST_ERROR) {
		free(buf);
		return o;
	}
	if(readlen < 1) {
		free(buf);
		return stt_create_eof(ctx->stt);
	}

	return stt_create_blob(ctx->stt, buf, readlen);
}

static STT_BUILTIN(sb_blob_slice)
{
	stt_obj_t *blob, *a_start, *a_len;
	size_t start, len, le;

	STT_ARG_BLOB(ctx->stt, blob);

	start = 0;
	len = blob->d.blob.size;

	if(parm->type == STT_ST_CONS) {
		STT_ARG_INT(ctx->stt, a_start);
		start = a_start->d.integer.value;

		if(parm->type == STT_ST_CONS) {
			STT_ARG_INT(ctx->stt, a_len);
			len = a_len->d.integer.value;
		}
	}

	le = blob->d.blob.size;

	if(start < 0) {
		start = le - (0 - start) % le;
		if(start == le) start = 0;
	}

	if(start >= le) return stt_create_blob(ctx->stt, NULL, 0);

	if(len > (le - start)) len = le - start;
	if(len < 0) len = le + len - start;
	if(len < 0) return stt_create_blob(ctx->stt, NULL, 0);

	return stt_create_blob_copy(ctx->stt, blob->d.blob.data + start, len);
}

/* find occurrence of one blob in another
 * returns offset, or -1
 * dir 0: from the left
 * dir 1: from the right
 */
int stt_blob_seek(unsigned char *needle, size_t needle_size, unsigned char *haystack, size_t haystack_size, int dir)
{
	unsigned char *haystack_end;

	if(needle_size > haystack_size) return -1;

	haystack_end = haystack + haystack_size - needle_size;

	if(!dir) {
		unsigned char *p = haystack;
		for(; p <= haystack_end; p ++)
			if(!memcmp(p, needle, needle_size)) return p - haystack;
		return -1;
	} else {
		unsigned char *p = haystack_end;
		for(; p >= haystack; p--)
			if(!memcmp(p, needle, needle_size)) return p - haystack;
		return -1;
	}
}

static STT_BUILTIN(sb_blob_find)
{
	stt_obj_t *needle, *haystack;
	int dir = 0;
	size_t begin = 0;
	long bgn;

	STT_ARG_BLOB(ctx->stt, needle);
	STT_ARG_BLOB(ctx->stt, haystack);

	if(parm->type == STT_ST_CONS) {
		stt_obj_t *rightp;
		STT_ARG_OBJ(ctx->stt, rightp);
		if(rightp->type == STT_ST_NIL) dir = 0; else dir = 1;

		if(parm->type == STT_ST_CONS) {
			STT_L_INT(ctx->stt, bgn);
			begin = bgn;
			if(begin < 0) return STT_INT(ctx->stt, -1);
			if(begin > haystack->d.blob.size) return STT_INT(ctx->stt, -1);
		} else {
			if(dir) begin = haystack->d.blob.size;
		}
	}

	if(!dir) {
		size_t r = stt_blob_seek(needle->d.blob.data, needle->d.blob.size,
				haystack->d.blob.data + begin,
				haystack->d.blob.size - begin, dir);
		if(r != -1) r += begin;
		return STT_INT(ctx->stt, r);
	} else {
		return STT_INT(ctx->stt, stt_blob_seek(needle->d.blob.data, needle->d.blob.size,
					haystack->d.blob.data, begin, dir));
	}
}

static STT_BUILTIN(sb_blob_cmp)
{
	stt_obj_t *blob1, *blob2;
	size_t l1, l2, l;
	int c;

	STT_ARG_BLOB(ctx->stt, blob1);
	STT_ARG_BLOB(ctx->stt, blob2);

	l1 = blob1->d.blob.size;
	l2 = blob2->d.blob.size;
	l = l1 < l2 ? l1 : l2;

	c = memcmp(blob1->d.blob.data, blob2->d.blob.data, l);
	if(c == 0) {
		if(l1 < l2) return STT_INT(ctx->stt, -1);
		else if(l1 > l2) return STT_INT(ctx->stt, 1);
	}
	return STT_INT(ctx->stt, c);
}

static STT_BUILTIN(sb_blob_string)
{
	stt_obj_t *blob;
	void *p;
	char *str;

	STT_ARG_BLOB(ctx->stt, blob);

	p = memchr(blob->d.blob.data, 0, blob->d.blob.size);
	if(p)
		return stt_create_error_with_obj(ctx->stt, STT_SE_PARM, "blob-string: 0 byte found in blob", STT_INT(ctx->stt, (char *)p - (char *)blob->d.blob.data));

	str = (char *) stt_malloc(blob->d.blob.size + 1);
	str[blob->d.blob.size] = 0;
	memcpy(str, blob->d.blob.data, blob->d.blob.size);

	return stt_create_string_allocd(ctx->stt, str);
}
