static STT_BUILTIN(sb_scar)
{
	stt_obj_t *name, *value;
	STT_ARG_CONS(ctx->stt, name);
	STT_ARG_OBJ(ctx->stt, value);

	if(data == 0)
		name->d.cons.car = value;
	else
		name->d.cons.cdr = value;

	return value;
}

/* print cons-dump */
stt_obj_t *stt_consdump(stt_varctx_t *ctx, stt_obj_t *obj)
{
	stt_obj_t *o;

	if(obj->type != STT_ST_CONS) {
		o = stt_stream_printf(ctx, "not a CONS: ");
		if(o->type == STT_ST_ERROR) return o;
		o = stt_printobj(ctx, obj);
		if(o->type == STT_ST_ERROR) return o;
		return stt_stream_stdout_write(ctx, '\n');
	}

	o = stt_stream_stdout_write(ctx, '(');
	if(o->type == STT_ST_ERROR) return o;
	if(obj->d.cons.car && obj->d.cons.car->type == STT_ST_CONS) o = stt_consdump(ctx, obj->d.cons.car);
		else o = stt_printobj(ctx, obj->d.cons.car);
	if(o->type == STT_ST_ERROR) return o;
	o = stt_stream_printf(ctx, " . ");
	if(o->type == STT_ST_ERROR) return o;
	if(obj->d.cons.cdr && obj->d.cons.cdr->type == STT_ST_CONS) o = stt_consdump(ctx, obj->d.cons.cdr);
		else o = stt_printobj(ctx, obj->d.cons.cdr);
	if(o->type == STT_ST_ERROR) return o;
	return stt_stream_stdout_write(ctx, ')');
}

static STT_BUILTIN(sb_consdump)
{
	stt_obj_t *obj;
	STT_ARG_OBJ(ctx->stt, obj);

	obj = stt_consdump(ctx, obj);
	if(obj->type == STT_ST_ERROR) return obj;
	return stt_stream_stdout_write(ctx, '\n');
}

static STT_BUILTIN(sb_cons)
{
	stt_obj_t *car, *cdr;
	STT_ARG_OBJ(ctx->stt, car);
	STT_ARG_OBJ(ctx->stt, cdr);

	return stt_create_cons(ctx->stt, car, cdr);
}

static STT_BUILTIN(sb_list)
{
	stt_obj_t *obj, *first, *cur;
	first = cur = NULL;

	while(stt_nextarg(ctx, &obj, &parm)) {
		if(obj->type == STT_ST_ERROR) return obj;

		if(!cur) first = cur = stt_create_cons(ctx->stt, obj, stt_create_nil(ctx->stt));
		else {
			cur->d.cons.cdr = stt_create_cons(ctx->stt, obj, stt_create_nil(ctx->stt));
			cur = cur->d.cons.cdr;
		}
	}

	if(!first) return stt_create_nil(ctx->stt);
	return first;
}

static STT_BUILTIN(sb_append)
{
	stt_obj_t *obj, *i, *first, *cur;
	first = cur = NULL;

	while(stt_nextarg(ctx, &obj, &parm)) {
		if(obj->type == STT_ST_ERROR) return obj;

		if(obj->type == STT_ST_CONS) {
			for(i = obj; i && i->type == STT_ST_CONS; i = i->d.cons.cdr) {
				if(!cur) first = cur = stt_create_cons(ctx->stt, i->d.cons.car, stt_create_nil(ctx->stt));
				else {
					cur->d.cons.cdr = stt_create_cons(ctx->stt, i->d.cons.car, stt_create_nil(ctx->stt));
					cur = cur->d.cons.cdr;
				}
			}
		} else if(obj->type != STT_ST_NIL) {
			if(!cur) return obj;
			cur->d.cons.cdr = obj;
		}
	}

	if(!first) return stt_create_nil(ctx->stt);
	return first;
}

static STT_BUILTIN(sb_conc)
{
	stt_obj_t *first;
	stt_obj_t *obj;
	stt_obj_t *cur;
	first = cur = NULL;

	while(stt_nextarg(ctx, &obj, &parm)) {
		if(obj->type == STT_ST_ERROR) return obj;

		if(obj->type == STT_ST_CONS) {
			if(!first) first = cur = obj;
			else cur->d.cons.cdr = obj;

			while(cur->d.cons.cdr->type == STT_ST_CONS)
				cur = cur->d.cons.cdr;
		} else if(obj->type != STT_ST_NIL) {
			return stt_create_error(ctx->stt, STT_SE_PARM, "Non-CONS argument given to conc.");
		}
	}

	return first ? first : stt_create_nil(ctx->stt);
}

static STT_BUILTIN(sb_car)
{
	stt_obj_t *obj;
	STT_ARG_OBJ(ctx->stt, obj);

	if(obj->type == STT_ST_NIL) return obj; /* car nil is nil */
	if(obj->type != STT_ST_CONS)
		return stt_create_error(ctx->stt, STT_SE_PARM, "Non-CONS parameter for car");

	return obj->d.cons.car;
}

static STT_BUILTIN(sb_cdr)
{
	stt_obj_t *obj;
	STT_ARG_OBJ(ctx->stt, obj);

	if(obj->type == STT_ST_NIL) return obj; /* cdr nil is nil */
	if(obj->type != STT_ST_CONS)
		return stt_create_error(ctx->stt, STT_SE_PARM, "Non-CONS parameter for cdr");

	return obj->d.cons.cdr;
}

static STT_BUILTIN(sb_length)
{
	stt_obj_t *obj;
	int l = 0;

	STT_ARG_OBJ(ctx->stt, obj);

	if(obj->type == STT_ST_STRING)
		return stt_create_int(ctx->stt, strlen(obj->d.string.value));
	else if(obj->type == STT_ST_BLOB)
		return stt_create_int(ctx->stt, obj->d.blob.size);

	for(; obj->type == STT_ST_CONS; obj = obj->d.cons.cdr)
		l++;

	return stt_create_int(ctx->stt, l);
}


static STT_BUILTIN(sb_maplist)
{
	stt_obj_t *lists[FUNC_MAXPARMS];
	stt_obj_t *obj, *call, *res = NULL, *calltail, *restail = NULL;
	stt_obj_t *quote, *func;
	stt_s_mark mark;
	int filter = (stt_intptr_t)data & 1;
	int car = (stt_intptr_t)data & 2;
	int destructive = (stt_intptr_t)data & 4;
	int i, lnum;

	STT_ARG_FUNCTION(ctx->stt, func);

	for(lnum = 0; lnum < FUNC_MAXPARMS && parm->type == STT_ST_CONS; lnum++) {
		stt_nextarg(ctx, &lists[lnum], &parm);
		if(lists[lnum]->type == STT_ST_ERROR) return lists[lnum];
	}
	if(!lnum)
		return stt_create_error(ctx->stt, STT_SE_PNUM, "Insufficient parameters for maplist/filter");

	quote = ctx->stt->quote_builtin;

	while(1) {
		for(i=0; i<lnum; i++)
			if(lists[i]->type != STT_ST_CONS) break;
		if(i < lnum) break;

		mark = stt_gc_prot_mark(ctx->stt);
		calltail = call = stt_create_cons(ctx->stt, func, stt_create_nil(ctx->stt));
		if(car)
			for(i=0; i<lnum; i++) {
				calltail->d.cons.cdr = stt_create_cons(ctx->stt, stt_create_cons(ctx->stt, quote, stt_create_cons(ctx->stt, lists[i]->d.cons.car, stt_create_nil(ctx->stt))), stt_create_nil(ctx->stt));
				calltail = calltail->d.cons.cdr;
			}
		else
			for(i=0; i<lnum; i++) {
				calltail->d.cons.cdr = stt_create_cons(ctx->stt, stt_create_cons(ctx->stt, quote, stt_create_cons(ctx->stt, lists[i], stt_create_nil(ctx->stt))), stt_create_nil(ctx->stt));
				calltail = calltail->d.cons.cdr;
			}

		obj = stt_eval(ctx, call);
		if(obj->type == STT_ST_ERROR) return obj;

		if(filter) {
			/* filterlist/filter */
			if(obj->type != STT_ST_NIL) {
				if(lnum > 1) {
					calltail = call = NULL;
					for(i=0; i<lnum; i++) {
						if(!calltail)
							calltail = call = stt_create_cons(ctx->stt, lists[i]->d.cons.car, stt_create_nil(ctx->stt));
						else {
							calltail->d.cons.cdr = stt_create_cons(ctx->stt, lists[i]->d.cons.car, stt_create_nil(ctx->stt));
							calltail = calltail->d.cons.cdr;
						}
					}
				} else call = lists[0]->d.cons.car;

				if(!restail)
					restail = res = stt_create_cons(ctx->stt, call, stt_create_nil(ctx->stt));
				else {
					restail->d.cons.cdr = stt_create_cons(ctx->stt, call, stt_create_nil(ctx->stt));
					restail = restail->d.cons.cdr;
				}
			}
		} else {
			if(destructive) {
				/* mapcan/mapcon */
				if(!restail || restail->type == STT_ST_NIL) {
					restail = res = obj;
					if(restail->type == STT_ST_CONS)
						while(restail->d.cons.cdr->type == STT_ST_CONS) restail = restail->d.cons.cdr;
				} else {
					if(restail->type != STT_ST_CONS)
						return stt_create_error_with_obj(ctx->stt, STT_SE_TYPE, "mapcan/mapcon user function returned non-cons", restail);
					restail->d.cons.cdr = obj;
					while(restail->d.cons.cdr->type == STT_ST_CONS)
						restail = restail->d.cons.cdr;
				}
			} else {
				/* maplist/mapcar */
				if(!restail)
					restail = res = stt_create_cons(ctx->stt, obj, stt_create_nil(ctx->stt));
				else {
					restail->d.cons.cdr = stt_create_cons(ctx->stt, obj, stt_create_nil(ctx->stt));
					restail = restail->d.cons.cdr;
				}
			}
		}

		for(i=0; i<lnum; i++)
			lists[i] = lists[i]->d.cons.cdr;

		stt_gc_prot_free(ctx->stt, mark, res);
	}

	return res ? res : stt_create_nil(ctx->stt);
}

