static STT_BUILTIN(sb_atom)
{
	stt_obj_t *obj;
	STT_ARG_OBJ(ctx->stt, obj);

	if(obj->type == STT_ST_CONS)
		return stt_create_nil(ctx->stt);
	return stt_create_t(ctx->stt);
}

static STT_BUILTIN(sb_functionp)
{
	stt_obj_t *obj;
	STT_ARG_OBJ(ctx->stt, obj);

	if(obj->type == STT_ST_FUNC || obj->type == STT_ST_BUILTIN)
		return stt_create_t(ctx->stt);
	return stt_create_nil(ctx->stt);
}

static STT_BUILTIN(sb_macrop)
{
	stt_obj_t *obj;
	STT_ARG_OBJ(ctx->stt, obj);

	if(obj->type == STT_ST_FUNC && (obj->d.func.macro == 1 || obj->d.func.macro == 2))
		return stt_create_t(ctx->stt);

	return stt_create_nil(ctx->stt);
}

static STT_BUILTIN(sb_symmacrop)
{
	stt_obj_t *obj;
	STT_ARG_OBJ(ctx->stt, obj);

	if(obj->type == STT_ST_FUNC && (obj->d.func.macro == 4 || obj->d.func.macro == 5))
		return stt_create_t(ctx->stt);

	return stt_create_nil(ctx->stt);
}

static STT_BUILTIN(sb_listp)
{
	stt_obj_t *obj;
	STT_ARG_OBJ(ctx->stt, obj);

	if(obj->type == STT_ST_CONS) {
		while(obj->d.cons.cdr->type == STT_ST_CONS)
			obj = obj->d.cons.cdr;
		return (obj->d.cons.cdr->type == STT_ST_NIL)
			? stt_create_t(ctx->stt) : stt_create_nil(ctx->stt);
	}

	if(obj->type == STT_ST_NIL) return stt_create_t(ctx->stt);
	return stt_create_nil(ctx->stt);
}

static STT_BUILTIN(sb_colonp)
{
	stt_obj_t *sym;
	STT_ARG_SYMBOL(ctx->stt, sym);

	if(sym->d.symbol.colon) return stt_create_t(ctx->stt); else return stt_create_nil(ctx->stt);
}

static STT_BUILTIN(sb_colonize)
{
	stt_obj_t *sym, *col, *o;
	STT_ARG_SYMBOL(ctx->stt, sym);
	STT_ARG_OBJ(ctx->stt, col);

	if(col->type == STT_ST_NIL) {
		if(!sym->d.symbol.colon) return sym;
		else return stt_create_symbol(ctx->stt, sym->d.symbol.name);
	}

	o = stt_alloc_obj(ctx->stt);
	o->type = STT_ST_SYMBOL;
	o->d.symbol.name = stt_strdup(sym->d.symbol.name);
	o->d.symbol.sdbm = sym->d.symbol.sdbm;
	o->d.symbol.id = sym->d.symbol.id;
	o->d.symbol.colon = 1;
	return o;
}

static STT_BUILTIN(sb_typep)
{
	stt_obj_t *obj;
	STT_ARG_OBJ(ctx->stt, obj);

	if(obj->type == (stt_type_t) data)
		return stt_create_t(ctx->stt);
	return stt_create_nil(ctx->stt);
}

static STT_BUILTIN(sb_weak_car)
{
	stt_obj_t *obj;
	STT_ARG_CONS(ctx->stt, obj);

	stt_weak_check(((stt_intptr_t) data) ? obj->d.cons.cdr : obj->d.cons.car);
	stt_weak_reference(obj, obj->serial, ((stt_intptr_t) data) ? &obj->d.cons.cdr : &obj->d.cons.car);
	return obj;
}
