#ifndef _ST_EVAL_H
#define _ST_EVAL_H

#include <stdio.h>
#include <estutter/stt_config.h>
#include <estutter/types.h>
#include <estutter/memory.h>

/* basic/internal error codes */
enum {
	STT_SE_UNDEF,	/* undefined symbol */
	STT_SE_WTF,		/* reserved */
	STT_SE_PARM,	/* invalid parameters */
	STT_SE_WFUNC,	/* invalid function */
	STT_SE_TYPE,	/* type mismatch */
	STT_SE_IPLIST,	/* non-symbol in parameter list of function */
	STT_SE_PNUM,	/* insufficient parameters */
	STT_SE_IO,		/* I/O error */
	STT_SE_PARSE,	/* Parse error */
	STT_SE_NFND,	/* Not found */
	STT_SE_LIMIT,	/* Recursion limit exceeded */
	STT_SE_MEM,		/* Memory allocation error */
	STT_SE_MATH,	/* Math exception */
	STT_SE_RET,		/* Return value */
	STT_SE_COMMA,	/* deprecated */
	STT_SE_COMMA_SPLICE, /* deprecated */
	STT_SE_STOPREAD, /* stop reading */

	STT_SE_LAST /* only a declaration, used by the reader.. */
};

char *stt_errorname(stt_ctx_t *stt, int id); /* returns name of error by id */
/* register new error code - returns new id.. name max 15 chars! */
int stt_regerrorcode(stt_ctx_t *stt, char *name);

#define stt_eval(ctx, obj) stt_eval_((ctx), (obj), 0)

/* evaluate expression
 * ctx is the variable context
 * obj is the object to be evaluated
 * tail is a boolean: whether the evaled object is in a tail position */
stt_obj_t *stt_eval_(stt_varctx_t *ctx, stt_obj_t *obj, const int tail);

/* user function executor
 * obj is the s-expression the call originates at
 * the tail flag is only used when evaluating macro results
 */
stt_obj_t *stt_funcexec(stt_varctx_t *ctx, stt_obj_t *obj, stt_obj_t *func, stt_obj_t *args, const int tail, int inhibit_macro_expansion);

/* load and evaluate from string */
stt_obj_t *stt_evalstr(stt_varctx_t *ctx, const char *script);

/* load and evaluate from fp */
stt_obj_t *stt_evalfp(stt_varctx_t *ctx, FILE *fp);

/* load and evaluate file */
stt_obj_t *stt_evalfile(stt_varctx_t *ctx, const char *name);

/* reset parser (line count, parsed expresion list) */
void stt_resetparser(void);

/* call (require 'system) */
stt_obj_t *stt_load_system(stt_varctx_t *ctx);

#endif
